/**
 * @file photo.c Galago Photo message processor
 *
 * Copyright (C) 2004-2005 Christian Hammond.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 */
#include "filter.h"

static DBusHandlerResult
photo_set_image(DBusConnection *dbus_conn, DBusMessage *message,
				GalagodCoCo *coco, GalagoPhoto *photo)
{
	DBusMessageIter iter;
	unsigned char *image_data;
	size_t image_len;

	dbus_message_iter_init(message, &iter);
	dbus_message_iter_get_byte_array(&iter, &image_data, &image_len);

	galago_photo_set_image_data(photo, image_data, image_len);

	dbus_free(image_data);

	return DBUS_HANDLER_RESULT_HANDLED;
}

static DBusHandlerResult
photo_path_message_func(DBusConnection *dbus_conn, DBusMessage *message,
						void *user_data)
{
	DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	GalagodStateData *data = (GalagodStateData *)user_data;

	if (dbus_message_get_type(message) == DBUS_MESSAGE_TYPE_SIGNAL)
		return result;

	galago_log_misc("%s: %s\n",
					dbus_message_get_interface(message),
					dbus_message_get_member(message));

	galago_context_push(data->context);

	if (dbus_message_is_method_call(message,
									GALAGO_DBUS_PHOTO_INTERFACE,
									"SetImage"))
	{
		result = photo_set_image(dbus_conn, message, data->coco,
								 data->u.photo);
	}
	else
	{
		galago_log_warning("Unhandled message!\n");
		galago_log_misc("\tobj_path = '%s'\n",
						dbus_message_get_path(message));
		galago_log_misc("\tinterface = '%s'\n",
						dbus_message_get_interface(message));
		galago_log_misc("\tmethod = '%s'\n",
						dbus_message_get_member(message));
	}

	galago_context_pop();

	return result;
}

void
galagod_photo_add_filter(GalagoPhoto *photo)
{
	galagod_filters_add(photo, photo_path_message_func);
}

void
galagod_photo_remove_filter(GalagoPhoto *photo)
{
	galagod_filters_remove(photo);
}
